package com.futu.goose.utils;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.futu.goose.course.pojo.Course;
import org.apache.http.HttpHost;
import org.elasticsearch.action.admin.indices.get.GetIndexRequest;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.support.master.AcknowledgedResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.indices.CreateIndexRequest;
import org.elasticsearch.client.indices.CreateIndexResponse;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.MatchAllQueryBuilder;
import org.elasticsearch.index.query.MatchQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Component
public class EsSyncUtils {
    private static final Logger logger = LoggerFactory.getLogger(EsSyncUtils.class);
    private RestHighLevelClient restHighLevelClient;
    private ObjectMapper objectMapper = new ObjectMapper();

    public EsSyncUtils() {
        this.restHighLevelClient = new RestHighLevelClient(
                RestClient.builder(new HttpHost("localhost", 9200, "http")));
    }

    public <T> void syncDataToEs(BaseMapper<T> mapper, String esIndex) {
        try {
            // 判断索引是否存在
            GetIndexRequest getIndexRequest = new GetIndexRequest();
            getIndexRequest.indices(esIndex);
            boolean indexExists = restHighLevelClient.indices().exists(getIndexRequest, RequestOptions.DEFAULT);
            if (indexExists) {
                // 删除索引
                AcknowledgedResponse deleteIndexResponse = restHighLevelClient.indices().delete(
                        new org.elasticsearch.action.admin.indices.delete.DeleteIndexRequest(esIndex),
                        RequestOptions.DEFAULT);
                if (!deleteIndexResponse.isAcknowledged()) {
                    logger.error("删除 Elasticsearch 索引 {} 失败", esIndex);
                }
            } else {
                logger.info("Elasticsearch 索引 {} 不存在，跳过删除步骤", esIndex);
            }
            // 从 MySQL 中查询所有数据
            List<T> dataList = mapper.selectList(null);

            // 创建批量请求
            BulkRequest bulkRequest = new BulkRequest();
            for (T data : dataList) {
                IndexRequest indexRequest = new IndexRequest(esIndex);
                String jsonString = objectMapper.writeValueAsString(data);
                indexRequest.source(jsonString, XContentType.JSON);
                bulkRequest.add(indexRequest);
            }

            // 执行批量请求
            BulkResponse bulkResponse = restHighLevelClient.bulk(bulkRequest, RequestOptions.DEFAULT);
            if (bulkResponse.hasFailures()) {
                logger.error("数据同步到 Elasticsearch 部分失败：{}", bulkResponse.buildFailureMessage());
            }
        } catch (IOException e) {
            logger.error("同步数据到 Elasticsearch 时发生 IO 异常", e);
        }
    }

    public Result searchDataByIndexAndName(String esIndex, String title) {
        try {
            SearchRequest searchRequest = new SearchRequest(esIndex);
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

            if (title == null || title.isEmpty()) {
                searchSourceBuilder.query(QueryBuilders.matchAllQuery());
            } else {
                searchSourceBuilder.query(QueryBuilders.matchQuery("title", title));
            }

            searchRequest.source(searchSourceBuilder);
            SearchResponse search = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits hits = search.getHits();

            if (hits.getTotalHits().value == 0) {
                return new Result(false, "没有查询到数据");
            }

            List<Course> courseList = new ArrayList<>();
            for (SearchHit hit : hits) {
                // 将JSON数据映射到Course实体类
                Course course = objectMapper.readValue(hit.getSourceAsString(), Course.class);
                courseList.add(course);
            }
            return new Result(true, "查询成功", courseList);
        } catch (Exception e) {
            logger.error("查询Elasticsearch数据失败", e);
            return new Result(false, "查询失败");
        }
    }
}    